/* eslint-env node, browser */
const fs = require('fs');
const path = require('path');
const fetchRecorderServer = require('fetch-recorder/fetchRecorder_server');
const browserScript = function(proxyServerAddress, redirects) {
    const readyMessage = 'fetchRecorder:serviceWorker:ready';
    const loadedMessage = 'fetchRecorder:serviceWorker:loaded';
    const proxyServerMarker = '<fetch_recorder_server>';
    redirects.forEach((redirect) => {
        if(redirect.replaceValue.startsWith(proxyServerMarker)) {
            redirect.replaceValue = redirect.replaceValue.replace(proxyServerMarker, proxyServerAddress);
        }
    });
    navigator.serviceWorker.register('/fetchRecorder_serviceWorker.js').then(({ active: serviceWorkerActive }) => {
        if(serviceWorkerActive) {
            navigator.serviceWorker.controller.postMessage({ redirects });
        }
    });
    navigator.serviceWorker.addEventListener('message', ({ data: message }) => {
        if(message === loadedMessage) {
            navigator.serviceWorker.controller.postMessage({ redirects });
        } else if(message === readyMessage) {
            const fetchRecorderReady = new Event(readyMessage);
            document.dispatchEvent(fetchRecorderReady);
        }
    });
};

module.exports = {
    generateScript() {
        const serverInfo = fetchRecorderServer.getInfo();
        const scriptText = `(${browserScript.toString()})('https://localhost:${serverInfo.port}', ${JSON.stringify(serverInfo.redirects)});`;
        const scriptPath = path.join(__dirname, 'fetchRecorder_generatedBrowserScript.js');
        fs.writeFileSync(scriptPath, scriptText);
        return scriptPath;
    }
};
