/* eslint-env worker */
const readyMessage = 'fetchRecorder:serviceWorker:ready';
const loadedMessage = 'fetchRecorder:serviceWorker:loaded';

let redirects = null;
let basicAuthValue = null;

self.addEventListener('install', (event) => {
    event.waitUntil(self.skipWaiting());
});
self.addEventListener('activate', (event) => {
    const claimClient = self.clients.claim();
    event.waitUntil(claimClient);
    claimClient.then(() => self.clients.matchAll({ includeUncontrolled: true, type: 'window' })).then((clients) => {
        clients.map((client) => client.postMessage(loadedMessage));
    });
});
self.addEventListener('message', ({ data }) => {
    if('redirects' in data) {
        redirects = data.redirects.reverse();
        self.clients.matchAll({ includeUncontrolled: true, type: 'window' }).then((clients) => {
            clients.map((client) => client.postMessage(readyMessage));
        });
    } else if('basicAuthValue' in data) {
        basicAuthValue = data.basicAuthValue;
    }
});
self.addEventListener('fetch', (event) => {
    if(redirects === null) {
        event.respondWith(fetch(event.request));
        return;
    }
    const req = event.request;
    let requestUrl = req.url;
    for(const redirect of redirects) {
        let matchPattern = redirect.matchPattern;
        if(redirect.useRegExp) {
            matchPattern = new RegExp(matchPattern);
        }
        requestUrl = requestUrl.replace(matchPattern, redirect.replaceValue);
        if(requestUrl !== req.url) {
            break;
        }
    }
    if(requestUrl !== req.url) {
        const parsedRequestUrl = new URL(requestUrl);
        const proxyTarget = `fetch-recorder-target=${encodeURIComponent((new URL(req.url)).origin)}`;
        parsedRequestUrl.search += requestUrl.search.length ? `&${proxyTarget}` : proxyTarget;
        const newUrl = parsedRequestUrl.toString();
        const newHeaders = new Headers();
        for(const [ key, value ] of req.headers.entries()) {
            newHeaders.set(key, value);
        }
        if(basicAuthValue !== null) {
            newHeaders.set('authorization', basicAuthValue);
        }
        const newRequest = new Request(newUrl, {
            credentials: req.credentials,
            headers: newHeaders,
            integrity: req.integrity,
            method: req.method,
            mode: req.mode,
            redirect: req.redirect,
            referrer: req.referrer
        });
        if(req.method !== 'GET' && req.method !== 'HEAD') {
            event.respondWith(req.clone().text().then((body) => {
                if(body !== '') {
                    newRequest.body = body;
                }
                return fetch(newRequest);
            }));
        } else {
            event.respondWith(fetch(newRequest));
        }
    } else {
        event.respondWith(fetch(event.request));
    }
});
